/* Prologue */
%expect 0
%error-verbose
%defines
%debug
%locations

%code provides
{
  /* C code. */
  int yylex ();
  void yyerror_ (const YYLTYPE* loc, const char* msg);
}

%code requires
{
#include <stdio.h>
#include <stdlib.h>

#define yyerror(Msg)                            \
  yyerror_(&yylloc, Msg)
}

%union
{
  int   ival;
  float fval;
};

%token TOK_PLUS  "+";
%token TOK_MINUS "-";
%token TOK_STAR  "*";
%token TOK_SLASH "/";

%token TOK_LPAREN "(";
%token TOK_RPAREN ")";

%token TOK_EOL    "end of line";

%token<ival> TOK_NUM    "number";
%type<ival> exp;


%printer { fprintf (stderr, "%d", $$); } <ival>;

%left "+" "-";
%left "*" "/";
%left UNARY;

%%
 /* Grammar */

lines:
  /* nothing */
| lines line
;

line:
        "end of line"  {}
| exp   "end of line"  { printf ("%d\n", $1); }
| error "end of line"  { yyerrok; }
;

exp:
  exp "+" exp    { $$ = $1 + $3; }
| exp "-" exp    { $$ = $1 - $3; }
| exp "*" exp    { $$ = $1 * $3; }
| exp "/" exp    { $$ = $1 / $3; }
| "number"       { $$ = $1; }
| "(" exp ")"    { $$ = $2; }
| "(" error ")"  { $$ = 666; }
| "-" exp %prec UNARY { $$ = -$2; }
;

%%
  /* Epilogue */

static unsigned errnums = 0;

void
yyerror_ (const YYLTYPE* loc, const char* msg)
{
  ++errnums;
  fprintf (stderr, "%d.%d-%d.%d: %s\n",
           loc->first_line,loc->first_column,
           loc->last_line,loc->last_column,
           msg);
}


int
main ()
{
  yydebug = !!getenv("YYDEBUG");
  yyparse ();
  return !!errnums;
}
