/* -*- C -*- */
%{
#include <stdlib.h>
#include "parse-calc.h"
char errorbuf[BUFSIZ];

# define YY_USER_ACTION                         \
  yylloc.last_column += yyleng;

# define LOCATION_STEP()                        \
 do {                                           \
  yylloc.first_line = yylloc.last_line;         \
  yylloc.first_column = yylloc.last_column;     \
 } while (0)

%}

%option nounput noyywrap

 /* Abbreviations. */
BLANK  [\t ]
DIGIT  [0-9]
NUMBER {DIGIT}+

%%

%{
  /* Inside the top of yylex. */
  LOCATION_STEP();
%}

  /* Rules. */

{NUMBER}        yylval.ival = strtol(yytext, 0, 10); return TOK_NUM;

"+"|"plus"      return TOK_PLUS;
"-"|"minus"     return TOK_MINUS;
"*"|"star"      return TOK_STAR;
"/"|"slash"     return TOK_SLASH;

"("|"lparen"    return TOK_LPAREN;
")"|"rparen"    return TOK_RPAREN;

\n              ++yylloc.last_line; yylloc.last_column = 1; return TOK_EOL;

{BLANK}+        LOCATION_STEP();

.      {
         snprintf(errorbuf, sizeof errorbuf,
                  "unexpected character: `%c'", yytext[0]);
         yyerror(errorbuf);
        }

%%
/* Epilogue. */
